﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SkyChatLibrary;
using System.Xml.Serialization;
using System.Xml;
using System.IO;

namespace SkyConsole
{
    class Program
    {
        public static SkyClient Client { get; set; }
        public static MySettings Settings { get; set; }
        public static XmlSerializer Serializer { get; set; }
        static void Main(string[] args)
        {
            Console.Title = "Sky Console by Mike";
            Serializer = new XmlSerializer(typeof(MySettings));
            Settings = new MySettings();
            LoadSettings();
            Client = new SkyClient(Settings.Connection);
            Client.EventClientSocketConnected += new SkyClient.DelegateClientSocketConnected(Client_EventClientSocketConnected);
            Client.EventClientSocketConnecting += new SkyClient.DelegateClientSocketConnecting(Client_EventClientSocketConnecting);
            Client.EventClientSocketDisconnected += new SkyClient.DelegateClientDisconnected(Client_EventClientSocketDisconnected);
            Client.EventClientSocketError += new SkyClient.DelegateClientSocketError(Client_EventClientSocketError);

            Client.EventClientNews += new SkyClient.DelegateClientNews(Client_EventClientNews);
            Client.EventClientChatUserTalks += new SkyClient.DelegateChatEvent(Client_EventClientChatUserTalks);
            Client.EventClientLogonResponse += new SkyClient.DelegateClientEventLogonResponse(Client_EventClientLogonResponse);
            Client.EventClientConnectedResponse += new SkyClient.DelegateClientEventConnectedResponse(Client_EventClientConnectedResponse);
            Client.EventClientChatInfo += new SkyClient.DelegateChatEvent(Client_EventClientChatInfo);

            Client.EventClientChatJoinChannel += new SkyClient.DelegateChatEvent(Client_EventClientChatJoinChannel);
            Client.EventClientChatUserWhisperReceived += new SkyClient.DelegateChatEvent(Client_EventClientChatUserWhisperReceived);
            Client.EventClientChatUserWhisperSent += new SkyClient.DelegateChatEvent(Client_EventClientChatUserWhisperSent);
            Client.EventClientChatUserShown += new SkyClient.DelegateChatEvent(Client_EventClientChatUserShown);

            Client.EventClientChatUserJoins += new SkyClient.DelegateChatEvent(Client_EventClientChatUserJoins);
            Client.EventClientChatUserLeaves += new SkyClient.DelegateChatEvent(Client_EventClientChatUserLeaves);

            Client.Connect();

            bool exit = false;
            while (!exit)
            {
                string result = Console.ReadLine();
                if (!string.IsNullOrEmpty(result))
                {
                    if (result.ToLower().StartsWith("/exit"))
                        exit = false;
                    else
                        Client.SendSkyChat(result);
                }
            }

        }

        static void Client_EventClientChatUserShown(ChatEventArgs e)
        {
            Console.WriteLine("[{0}] SHOWN {1} {2} {3}", DateTime.Now.ToShortTimeString(), e.Username, e.UserFlags, e.Ping);
        }

        static void Client_EventClientChatUserLeaves(ChatEventArgs e)
        {
            Console.WriteLine("[{0}] LEAVE {1}", DateTime.Now.ToShortTimeString(), e.Username);
        }

        static void Client_EventClientChatUserJoins(ChatEventArgs e)
        {
            Console.WriteLine("[{0}] JOIN {1} {2} {3}", DateTime.Now.ToShortTimeString(), e.Username, e.UserFlags, e.Ping);
        }

        static void Client_EventClientChatUserWhisperSent(ChatEventArgs e)
        {
            Console.WriteLine("[{0}] TO {1}: {2}", DateTime.Now.ToShortTimeString(), e.Username, e.Message);
        }

        static void Client_EventClientChatUserWhisperReceived(ChatEventArgs e)
        {
            Console.WriteLine("[{0}] FROM {1}: {2}", DateTime.Now.ToShortTimeString(), e.Username, e.Message);
        }

        static void Client_EventClientChatJoinChannel(ChatEventArgs e)
        {
            Console.WriteLine("[{0}] Joined Channel {{1}}", DateTime.Now.ToShortTimeString(), e.Message);
        }

        static void Client_EventClientChatInfo(ChatEventArgs e)
        {
            Console.WriteLine("[{0}] INFO {1}", DateTime.Now.ToShortTimeString(), e.Message);
        }

        static void Client_EventClientConnectedResponse(SkyRequestConnectEx result)
        {
            Console.WriteLine("Client connected response: {0}", result);
        }

        static void Client_EventClientLogonResponse(SkyLogin result, string Username)
        {
            Console.WriteLine("Client logon response: {0}", result);
        }

        static void Client_EventClientChatUserTalks(ChatEventArgs e)
        {
            Console.WriteLine("[{0}] CHAT {1}: {2}",DateTime.Now.ToShortTimeString(), e.Username, e.Message);
        }

        static void Client_EventClientNews(string news)
        {
            Console.WriteLine("[{0}] Server News {0}", DateTime.Now.ToShortTimeString(), news);
        }

        static void Client_EventClientSocketError(System.Net.Sockets.SocketException ex)
        {
            Console.WriteLine("Client socket error {0}", ex.ErrorCode);
        }

        static void Client_EventClientSocketDisconnected()
        {
            Console.WriteLine("Client socket has disconnected.");
        }

        static void Client_EventClientSocketConnecting()
        {
            Console.WriteLine("Client socket is connecting.");
        }

        static void Client_EventClientSocketConnected()
        {
            Console.WriteLine("Client socket has connected.");
        }

        static void LoadSettings()
        {
            if (!File.Exists("Config.xml"))
            {
                CreateSettings();
                return;
            }

            try
            {
                Settings.Connection = new SkyClientSettings();
                using (StreamReader r = new StreamReader("Config.xml"))
                {
                    Settings = (MySettings)Serializer.Deserialize(r);
                }
            }
            catch
            {
                CreateSettings();
            }

        }

        static void CreateSettings()
        {
            Settings.Connection = new SkyClientSettings();
            Settings.Connection.Username = "SkyConsole";
            Settings.Connection.Password = "ConsolePW";
            Settings.Connection.Email = "skyconsole@mikeownage.com";
            Settings.Connection.Channel = "Public Chat";
            Settings.Connection.Server = "skychat.theskyproject.net";
            Settings.Connection.Port = 9732;

            using (StreamWriter w = new StreamWriter("Config.xml"))
            {
                Serializer.Serialize(w, Settings);
            }
        }
    }

    [XmlRoot("Settings")]
    public class MySettings
    {
        public SkyClientSettings Connection { get; set; }
    }
}
